import re
from django.db import models
from django.utils.safestring import mark_safe

PARAMETER_TYPES = [
    ("STR", "String"),
    ("INT", "Integer"),
    ("DNS", "Valid DNS name"),
]

TEMPLATE_ICONS = [
    ("cm.svg", "ConfigMap"),
    ("crb.svg", "ClusterRoleBinding"),
    ("crd.svg", "CustomResourceDefinition"),
    ("c-role.svg", "ClusterRole"),
    ("cronjob.svg", "CronJob"),
    ("deploy.svg", "Deployment"),
    ("ds.svg", "DaemonSet"),
    ("ep.svg", "Endpoint"),
    ("group.svg", "Group"),
    ("hpa.svg", "HorizontalPodAutoscaler"),
    ("ing.svg", "Ingress"),
    ("job.svg", "Job"),
    ("limits.svg", "LimitRange"),
    ("netpol.svg", "NetworkPolicy"),
    ("ns.svg", "Namespace"),
    ("pod.svg", "Pod"),
    ("psp.svg", "PodSecurityPolicy"),
    ("pvc.svg", "PermanentVolumeClaim"),
    ("pv.svg", "PermanentVolume"),
    ("quota.svg", "ResourceQuota"),
    ("rb.svg", "RoleBinding"),
    ("role.svg", "Role"),
    ("rs.svg", "ReplicaSet"),
    ("sa.svg", "ServiceAccount"),
    ("sc.svg", "StorageClass"),
    ("secret.svg", "Secret"),
    ("sts.svg", "StatefulSet"),
    ("svc.svg", "Service"),
    ("user.svg", "User"),
    ("vol.svg", "Volume"),
    ("k8s.svg", "Kubernetes"),
]



def needs_yaml_quotes(value):
    return (
        not value or
        re.search("[^a-zA-Z0-9.]", value) or
        re.match("^[0-9]+$", value) or
        value.lower() in ["yes", "no"]
    )

def safe_str(value):
    value = str(value)

    def enc(match):
        char = match.group(0)
        n = ord(char)
        if n < 255:
            return "\\x%02x" % n
        else:
            return "\\u%04x" % n

    escaped = re.sub("[^a-zA-Z0-9-.:]", enc, value)
    if needs_yaml_quotes(escaped):
        escaped = '"' + escaped + '"'
    return mark_safe(escaped)

def safe_dns(value):
    if len(value) > 253:
        raise ValueError("Name too long")
    if not re.match("^[a-zA-Z0-9.-]+$", value):
        raise ValueError("Invalid chars")
    if not re.search("^[a-zA-Z0-9]", value):
        raise ValueError("Does not start with alphanumeric")
    if not re.search("[a-zA-Z0-9]$", value):
        raise ValueError("Does not end with alphanumeric")
    return safe_str(value)

PARAMETER_EVALS = {
    "STR": safe_str,
    "INT": int,
    "DNS": safe_dns,
}
    
class KubeTemplate(models.Model):
    """Kubernetes manifest template"""
    name = models.CharField(max_length=255)
    title = models.CharField(max_length=25)
    description = models.TextField()
    icon = models.CharField(max_length=32, choices=TEMPLATE_ICONS, default="Kubernetes")
    template = models.TextField()
    listed = models.BooleanField(default=False)

    def __str__(self):
        return self.name

class Parameter(models.Model):
    """Parameter used in templates"""
    name = models.CharField(max_length=64)
    default = models.CharField(max_length=255, blank=True)
    is_required = models.BooleanField(default=True)
    template = models.ForeignKey(
            KubeTemplate,
            on_delete=models.CASCADE,
            related_name="parameters",
    )
    type = models.CharField(
        max_length=3,
        default=PARAMETER_TYPES[0],
        choices=PARAMETER_TYPES
    )

    def eval(self, value):
        return PARAMETER_EVALS[self.type](value)

    def __str__(self):
        return f"{self.template.name}.{self.name}"

    def html_type(self):
        """Return the HTML input type for the parameter"""
        if self.type in ["INT"]:
            return "number"
        return "text"

